from django.core.files.storage import (
    FileSystemStorage as DjangoFileSystemStorage,
)


class FileSystemStorage(DjangoFileSystemStorage):
    def __init__(self, dsn):
        base_url = dsn.args.get("url")
        super().__init__(location=str(dsn.path), base_url=base_url)
        if base_url is None:
            self.base_url = None



from django.test import TestCase

from django_storage_url import get_storage
from django_storage_url.backends.s3 import S3Storage


class S3BackendTestCase(TestCase):
    def test_get_s3_storage(self):
        storage = get_storage(
            "s3://"
            "access_key_id:secret_access_key@"
            "bucket.example.com.s3.amazonaws.com"
            "/?auth=s3v4&domain=custom.com"
        )

        self.assertIsInstance(storage, S3Storage)

        self.assertEqual(storage.access_key, "access_key_id")
        self.assertEqual(storage.secret_key, "secret_access_key")
        self.assertEqual(storage.bucket_name, "bucket.example.com")
        self.assertEqual(storage.endpoint_url, "https://s3.amazonaws.com/")
        self.assertEqual(storage.addressing_style, "path")
        self.assertEqual(storage.signature_version, "s3v4")
        self.assertEqual(storage.location, "")
        self.assertEqual(storage.region_name, None)
        self.assertEqual(storage.custom_domain, "custom.com")
        self.assertEqual(storage.object_parameters, {"ACL": "public-read"})
        self.assertEqual(storage.querystring_auth, False)
        self.assertEqual(storage.url_protocol, "https:")
        self.assertEqual(storage.base_url, "https://custom.com/")



from django.conf import settings
from django.utils.functional import LazyObject

import furl

from .backends import get_storage_class
from .backends.not_implemented import NotImplementedStorage


class _DSNConfiguredStorage(LazyObject):
    def _setup(self):
        dsn = getattr(settings, self._setting_name, None)
        if not dsn:
            args = []
            storage_class = NotImplementedStorage
        else:
            url = furl.furl(dsn)
            storage_class = get_storage_class(url.scheme)
            args = [url]

        # Django >= 1.9 now knows about LazyObject and sets them up before
        # serializing them. To work around this behavior, the storage class
        # itself needs to be deconstructible.
        storage_class = type(
            storage_class.__name__,
            (storage_class,),
            {
                "deconstruct": self._deconstructor,
                "__reduce__": self._pickler,
                "__module__": storage_class.__module__,
            },
        )
        self._wrapped = storage_class(*args)


def dsn_configured_storage_class(setting_name):
    path = "{}.{}".format(
        dsn_configured_storage.__module__, dsn_configured_storage.__name__
    )
    return type(
        "DSNConfiguredStorage",
        (_DSNConfiguredStorage,),
        {
            "_setting_name": setting_name,
            "_deconstructor": lambda self: (path, [setting_name], {}),
            "_pickler": lambda self: (dsn_configured_storage, (setting_name,)),
        },
    )


def dsn_configured_storage(setting_name):
    return dsn_configured_storage_class(setting_name)()


def get_storage(dsn):
    url = furl.furl(dsn)
    storage_class = get_storage_class(url.scheme)
    return storage_class(url)



from django.core.files.storage import (
    get_storage_class as django_get_storage_class,
)


SCHEMES = {
    "az": "django_storage_url.backends.az.AzureStorage",
    "file": "django_storage_url.backends.file.FileSystemStorage",
    "s3": "django_storage_url.backends.s3.S3Storage",
}


def register_storage_class(scheme, backend_path):
    SCHEMES[scheme] = backend_path


def get_storage_class(scheme):
    return django_get_storage_class(SCHEMES[scheme])



from django.core.files.storage import Storage


class NotImplementedStorage(Storage):
    base_url = "/__not_implemented__/"

    def open(self, name, mode="rb"):
        raise NotImplementedError

    def save(self, name, content, max_length=None):
        raise NotImplementedError

    def get_valid_name(self, name):
        raise NotImplementedError

    def get_available_name(self, name, max_length=None):
        raise NotImplementedError
